{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2022 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/details/base.html" %}

{%- set title = community.metadata.title + _(" requests") -%}
{% set active_community_header_menu_item = 'requests' %}

{%- block javascript %}
{{ super() }}
{{ webpack['invenio-communities-requests.js'] }}
{%- endblock javascript %}


{%- block page_body %}
{{ super() }}
  {%- block requests_body %}
  <div
    id="communities-request-search-root"
    class="rel-mt-2"
    data-community='{{ community | tojson }}'
    data-invenio-search-config='{{ search_app_communities_requests_config(app_id="communities-requests-search", endpoint=community.links.requests) | tojson }}'>
  </div>
  {%- endblock requests_body %}
{%- endblock page_body %}
